import sqlite3

from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String, Text, UniqueConstraint, create_engine
from sqlalchemy.orm import backref, declarative_base, relationship, sessionmaker
from sqlalchemy.sql import func
from sqlalchemy.sql import default_comparator

from server.config import ServerConf

Base = declarative_base()


class Contact(Base):
    """
    Таблица контактов, связывающая таблицу пользователей в отношении "многие ко многим"
    """

    __tablename__ = "contact"

    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey("user.id", onupdate="CASCADE", ondelete="CASCADE"), nullable=True)
    contact_id = Column(Integer, ForeignKey("user.id", onupdate="CASCADE", ondelete="CASCADE"), nullable=True)
    is_active = Column(Boolean, default=True, index=True)

    __table_args__ = (UniqueConstraint(user_id, contact_id),)

    user = relationship("User", backref=backref("user", uselist=False), foreign_keys=[user_id], viewonly=True)
    contact = relationship("User", backref=backref("contact", uselist=False), foreign_keys=[contact_id], viewonly=True)


class User(Base):
    """
    Таблица для хранения зарегистрированных на сервере пользователей
    """

    __tablename__ = "user"

    id = Column(Integer, primary_key=True)
    username = Column(String, index=True, unique=True)
    password = Column(String)
    status = Column(Text)
    is_active = Column(Boolean, default=False, index=True)
    contacts = relationship(
        "User",
        secondary="contact",
        primaryjoin=id == Contact.user_id,
        secondaryjoin=id == Contact.contact_id,
    )


class History(Base):
    """
    Таблица с историей входов пользователей на сервер
    """

    __tablename__ = "history"

    id = Column(Integer, primary_key=True)
    username = Column(String, nullable=False)
    ip_address = Column(String, nullable=False)
    login_timestamp = Column(DateTime, default=func.now())


def init_db():
    """Функция инициализации сессии ORM и базы данных при её отсутствии"""
    conn_str = ServerConf.DB_CONFIG["URL"]
    engine = create_engine(conn_str, future=True, echo=False, pool_recycle=7200)
    if not ServerConf.DB_PATH.exists():
        Base.metadata.create_all(engine)
    Session = sessionmaker(bind=engine)
    session = Session()
    return session    



import pathlib
import platform
import sys
from threading import Thread
from subprocess import Popen

from PyQt6 import QtCore, QtWidgets

from server.config import ServerConf
from server.gui.clients_window import Ui_ClientsWindow
from server.gui.history_window import Ui_HistoryWindow
from server.gui.main_window import Ui_MainWindow
from server.storage import ServerStorage
from server.transport import JIMServer


class Application:
    """
    Класс, задающий поведение главного окна графического интерфейса сервера
    """

    def __init__(self) -> None:
        self.app = QtWidgets.QApplication(sys.argv)
        self.main_window = QtWidgets.QMainWindow()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self.main_window)
        self.ui.lineEditIP.setText(ServerConf.DEFAULT_LISTENER_ADDRESS)
        self.ui.lineEditPort.setText(str(ServerConf.DEFAULT_PORT))
        self.clients_window = ClientsWindow()
        self.history_window = HistoryWindow()
        self.server_storage = ServerStorage()
        self.add_user_dialog = AddUserDialog()
        self._create_bindings()
        self.server_task = None

    def run(self):
        """Запускает отображение интерфейса"""
        self.main_window.show()
        ret_code = self.app.exec()
        self._on_click_stop_server()
        sys.exit(ret_code)

    def _create_bindings(self):
        self.add_user_dialog.accept_creds.connect(self._accept_creds_slot)
        self.ui.pushButtonAddContact.clicked.connect(self._on_click_add_user)
        self.ui.pushButtonDeleteContact.clicked.connect(self._on_click_del_user)
        self.ui.pushButtonClients.clicked.connect(self._on_click_show_clients)
        self.ui.pushButtonHistory.clicked.connect(self._on_click_show_history)
        self.ui.pushButtonStartServer.clicked.connect(self._on_click_start_server)

    def _on_click_start_server(self):
        if not self.server_task:
            self._start_server()
            self.ui.pushButtonStartServer.setText("Остановить сервер")
            self.ui.pushButtonStartServer.clicked.connect(self._on_click_stop_server)

    def _on_click_stop_server(self):
        if self.server_task:
            self.server_task.kill()
            self.server_task.terminate()
            self.server_task = None
            self.ui.pushButtonStartServer.setText("Запустить сервер")
            self.ui.pushButtonStartServer.clicked.connect(self._on_click_start_server)

    def _start_server(self):
        address = self.ui.lineEditIP.text().strip()
        port = self.ui.lineEditPort.text().strip()
        bin_path = pathlib.Path().resolve() / 'run_server_cli'
        if not bin_path.exists():
            server_path = pathlib.Path().resolve() / 'src/server/run_server_cli.py'
            if not server_path.exists():
                server_path = pathlib.Path().resolve() / 'run_server_cli.py'
            server_cmd = f'python {server_path} -a {address} -p {port}'
        else:
            server_cmd = f'{bin_path} -a {address} -p {port}'
        if platform.system().lower() == "windows":
            self.server_task = Popen(server_cmd, creationflags=subprocess.CREATE_NEW_CONSOLE)  # type: ignore
        else:
            self.server_task = Popen(server_cmd, executable='/bin/bash', shell=True)

    def _on_click_show_history(self):
        self.history_window.show()

    def _on_click_show_clients(self):
        self.clients_window.show()

    def _on_click_add_user(self):
        self.add_user_dialog.clear_data()
        self.add_user_dialog.show()

    def _accept_creds_slot(self, username: str, password: str):
        if username and password:
            if not self.server_storage.check_user_exists(username=username):
                self.server_storage.add_user(username=username, password=password)
                self._show_standard_notification(
                    title="Успешно", info=f'Пользователь "{username}" добавлен', is_warning=False
                )
            else:
                self._show_standard_notification(info="Пользовтель с таким именем уже существует")
        else:
            self._show_standard_notification(info='Поля "имя пользователя" и "пароль" не должны быть пустыми')

    def _on_click_del_user(self):
        username, ok = QtWidgets.QInputDialog.getText(self.main_window, "Удаление пользователя", "Введите имя:")
        if ok:
            if self.server_storage.check_user_exists(username=username):
                res = self.server_storage.remove_user(username=username)
                if res:
                    self._show_standard_notification(
                        title="Успешно", info=f'Пользователь "{username}" удален', is_warning=False
                    )
                else:
                    self._show_standard_notification(info="Ошибка удаления пользователя")
            else:
                self._show_standard_notification(info="Пользователь с таким именем отсутсвует")

    def _show_standard_notification(self, info: str, title: str = "Ошибка", is_warning: bool = True):
        msg = QtWidgets.QMessageBox()
        if is_warning:
            msg.setWindowIcon(self.app.style().standardIcon(QtWidgets.QStyle.StandardPixmap.SP_MessageBoxWarning))
        else:
            msg.setWindowIcon(self.app.style().standardIcon(QtWidgets.QStyle.StandardPixmap.SP_MessageBoxInformation))
        msg.setWindowTitle(title)
        msg.setInformativeText(info)
        msg.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok)
        msg.exec()


class HistoryWindow:
    """
    Класс окна просмотра истории входов клиентов
    """

    def __init__(self) -> None:
        self.server_storage = ServerStorage()
        self.window = QtWidgets.QWidget()
        self.ui = Ui_HistoryWindow()
        self.ui.setupUi(self.window)

    def show(self):
        self.update_data()
        self.window.show()
        self.ui.tableWidget.show()

    def update_data(self):
        users_history = self.server_storage.get_users_history()
        header_labels = ["Имя пользователя", "IP адрес", "Дата и время входа"]
        self.ui.tableWidget.setRowCount(len(users_history))
        self.ui.tableWidget.setColumnCount(len(header_labels))
        self.ui.tableWidget.setHorizontalHeaderLabels(header_labels)
        header = self.ui.tableWidget.horizontalHeader()
        header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
        header.setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
        header.setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeMode.Stretch)

        for index, (username, ip_address, login_timestamp) in enumerate(users_history):
            item_user = QtWidgets.QTableWidgetItem(username)
            item_ip = QtWidgets.QTableWidgetItem(ip_address)
            item_timestamp = QtWidgets.QTableWidgetItem(str(login_timestamp.replace(microsecond=0)))
            self.ui.tableWidget.setItem(index, 0, item_user)
            self.ui.tableWidget.setItem(index, 1, item_ip)
            self.ui.tableWidget.setItem(index, 2, item_timestamp)


class ClientsWindow:
    """
    Класс окна просмотра активных клиентов
    """

    def __init__(self) -> None:
        self.server_storage = ServerStorage()
        self.window = QtWidgets.QWidget()
        self.ui = Ui_ClientsWindow()
        self.ui.setupUi(self.window)

    def show(self):
        self.update_data()
        self.window.show()
        self.ui.tableWidget.show()

    def update_data(self):
        active_users = self.server_storage.get_active_users()
        header_labels = ["Имя пользователя"]
        self.ui.tableWidget.setRowCount(len(active_users))
        self.ui.tableWidget.setColumnCount(len(header_labels))
        self.ui.tableWidget.setHorizontalHeaderLabels(header_labels)
        header = self.ui.tableWidget.horizontalHeader()
        header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)
        for index, user in enumerate(active_users):
            item_user = QtWidgets.QTableWidgetItem(user)
            self.ui.tableWidget.setItem(index, 0, item_user)


class AddUserDialog(QtWidgets.QDialog):
    """
    Класс диалога добавления пользователя
    """

    accept_creds = QtCore.pyqtSignal(str, str)

    def __init__(self, parent=None):
        super(AddUserDialog, self).__init__(parent)
        self.labelUsername = QtWidgets.QLabel(self)
        self.labelUsername.setText("Имя пользователя:")
        self.usernameLineEdit = QtWidgets.QLineEdit(self)
        self.labelPasswd1 = QtWidgets.QLabel(self)
        self.labelPasswd1.setText("Пароль:")
        self.passwd1LineEdit = QtWidgets.QLineEdit(self)
        self.passwd2LineEdit = QtWidgets.QLineEdit(self)
        self.labelPasswd2 = QtWidgets.QLabel(self)
        self.labelPasswd2.setText("Повторите пароль:")
        self.passwd1LineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
        self.passwd2LineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
        self.buttonSaveUser = QtWidgets.QPushButton("Сохранить")
        self.buttonSaveUser.clicked.connect(self.handle_registration)
        layout = QtWidgets.QVBoxLayout(self)
        layout.addWidget(self.labelUsername)
        layout.addWidget(self.usernameLineEdit)
        layout.addWidget(self.labelPasswd1)
        layout.addWidget(self.passwd1LineEdit)
        layout.addWidget(self.labelPasswd2)
        layout.addWidget(self.passwd2LineEdit)
        layout.addWidget(self.buttonSaveUser)

    def handle_registration(self):
        if self.passwd1LineEdit.text() == self.passwd2LineEdit.text():
            username = self.usernameLineEdit.text()
            password1 = self.passwd1LineEdit.text()
            password2 = self.passwd2LineEdit.text()
            if password1 == password2:
                self.accept_creds.emit(username, password1)
                self.close()
        else:
            QtWidgets.QMessageBox.warning(self, "Ошибка", "Пароли не совпадают")

    def clear_data(self):
        self.usernameLineEdit.clear()
        self.passwd1LineEdit.clear()
        self.passwd2LineEdit.clear()



import binascii
import hashlib

from sqlalchemy.exc import NoResultFound

from server.config import ServerConf
from server.model import Contact, History, User, init_db


class ServerStorage:
    """
    Класс-обертка над ORM для взаимодействия сервера с базой данных
    """

    def __init__(self) -> None:
        self.session = init_db()
        self.set_all_users_inactive()

    def check_user_exists(self, username: str):
        try:
            self.session.query(User).filter_by(username=username).one()
            return True
        except NoResultFound:
            return False

    def check_user_auth(self, username: str, password: str):
        if self.check_user_exists:
            password = self.make_passwd_hash(username=username, password=password)
            db_password = self.get_user_password_hash(username=username)
            if password == db_password:
                return True
        return False

    def remove_user(self, username: str):
        try:
            user = self.session.query(User).filter_by(username=username).one()
        except NoResultFound:
            return False
        else:
            self.session.delete(user)
            self.session.commit()
            return True

    def make_passwd_hash(self, username: str, password: str):
        b_salt = username.encode(ServerConf.ENCODING)
        b_password = password.encode(ServerConf.ENCODING)
        pswd_hash = hashlib.pbkdf2_hmac("sha256", password=b_password, salt=b_salt, iterations=10000)
        return binascii.hexlify(pswd_hash).decode(ServerConf.ENCODING)

    def get_user_password_hash(self, username: str):
        try:
            user = self.session.query(User).filter_by(username=username).one()
            return user.password
        except NoResultFound:
            return None

    def register_user_login(self, username, ip_address):
        history = History(username=username, ip_address=ip_address)
        self.session.add(history)
        self.session.commit()

    def add_user(self, username, password):
        password = self.make_passwd_hash(username=username, password=password)
        user = User(username=username, password=password)
        self.session.add(user)
        self.session.commit()

    def change_user_status(self, username: str, is_active: bool, status: str = ""):
        user = self.session.query(User).filter_by(username=username).one()
        user.is_active = is_active
        if status:
            user.status = status
        self.session.add(user)
        self.session.commit()

    def get_active_users(self):
        query = self.session.query(User).filter_by(is_active=True).all()
        return [entry.username for entry in query]

    def get_users_history(self):
        query = self.session.query(History).all()
        return [(entry.username, entry.ip_address, entry.login_timestamp) for entry in query]

    def set_all_users_inactive(self):
        for user in self.get_active_users():
            self.change_user_status(user, is_active=False)

    def get_user_contacts(self, username: str):
        try:
            user = self.session.query(User).filter_by(username=username).one()
            contacts = self.session.query(Contact).filter_by(user_id=user.id, is_active=True).all()
        except NoResultFound:
            return []
        else:
            return [contact.contact.username for contact in contacts]

    def delete_contact(self, username: str, contact_name: str):
        try:
            user = self.session.query(User).filter_by(username=username).one()
            contact = self.session.query(User).filter_by(username=contact_name).one()
            self.session.query(Contact).filter_by(user_id=user.id, contact_id=contact.id).update({"is_active": False})
            self.session.commit()
            return True
        except NoResultFound:
            return False

    def add_contact(self, username: str, contact_name: str):
        try:
            user = self.session.query(User).filter_by(username=username).one()
            contact = self.session.query(User).filter_by(username=contact_name).one()
            user.contacts.append(contact)
            self.session.query(Contact).filter_by(user_id=user.id, contact_id=contact.id).update({"is_active": True})
            self.session.commit()
            return True
        except NoResultFound:
            return False



# Form implementation generated from reading ui file './server/gui/server_gui.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(300, 273)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_2.setObjectName("formLayout_2")
        self.labelIP = QtWidgets.QLabel(self.groupBox)
        self.labelIP.setObjectName("labelIP")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.labelIP)
        self.lineEditIP = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditIP.setObjectName("lineEditIP")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lineEditIP)
        self.labelPort = QtWidgets.QLabel(self.groupBox)
        self.labelPort.setObjectName("labelPort")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.labelPort)
        self.lineEditPort = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditPort.setObjectName("lineEditPort")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lineEditPort)
        self.verticalLayout.addWidget(self.groupBox)
        self.pushButtonStartServer = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonStartServer.setObjectName("pushButtonStartServer")
        self.verticalLayout.addWidget(self.pushButtonStartServer)
        self.pushButtonAddContact = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonAddContact.setObjectName("pushButtonAddContact")
        self.verticalLayout.addWidget(self.pushButtonAddContact)
        self.pushButtonDeleteContact = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonDeleteContact.setObjectName("pushButtonDeleteContact")
        self.verticalLayout.addWidget(self.pushButtonDeleteContact)
        self.pushButtonClients = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonClients.setObjectName("pushButtonClients")
        self.verticalLayout.addWidget(self.pushButtonClients)
        self.pushButtonHistory = QtWidgets.QPushButton(self.centralwidget)
        self.pushButtonHistory.setObjectName("pushButtonHistory")
        self.verticalLayout.addWidget(self.pushButtonHistory)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "JIM-сервер"))
        self.groupBox.setTitle(_translate("MainWindow", "Настройки сервера"))
        self.labelIP.setText(_translate("MainWindow", "IP адрес"))
        self.labelPort.setText(_translate("MainWindow", "Порт"))
        self.pushButtonStartServer.setText(_translate("MainWindow", "Запуск сервера"))
        self.pushButtonAddContact.setText(_translate("MainWindow", "Добавить пользователя"))
        self.pushButtonDeleteContact.setText(_translate("MainWindow", "Удалить пользователя"))
        self.pushButtonClients.setText(_translate("MainWindow", "Просмотр списка клиентов"))
        self.pushButtonHistory.setText(_translate("MainWindow", "Просмотр истории клиентов"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec())



import os
import pathlib

from common.config import CommonConf


class ServerConf(CommonConf):
    """
    Настройки сервера по умолчанию
    """

    ROOT_DIR = pathlib.Path().resolve()
    LOGS_DIR = ROOT_DIR / "logs"
    DATA_DIR_NAME = "data"
    DATA_DIR = ROOT_DIR / DATA_DIR_NAME
    DB_NAME = "jim_server_db.sqlite"
    DB_PATH = DATA_DIR / DB_NAME
    DB_CONFIG = {
        "TEST_URL": "sqlite:///:memory:",
        "URL": f"sqlite:///{DB_PATH}",
        "USER": "",
        "PSWD": "",
    }

    DEFAULT_LISTENER_ADDRESS = "0.0.0.0"
    MAX_CONNECTIONS = 5
    MAIN_LOG_FILE_PATH = LOGS_DIR / "server.error.log"
    CALL_LOG_FILE_PATH = LOGS_DIR / "server.calls.log"


os.makedirs(ServerConf.LOGS_DIR, exist_ok=True)
os.makedirs(ServerConf.DATA_DIR, exist_ok=True)



"""initial

Revision ID: bdb20895b216
Revises: 
Create Date: 2023-01-04 07:48:59.741883

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "bdb20895b216"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "history",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("username", sa.String(), nullable=False),
        sa.Column("ip_address", sa.String(), nullable=False),
        sa.Column("login_timestamp", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "user",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("username", sa.String(), nullable=True),
        sa.Column("password", sa.String(), nullable=True),
        sa.Column("status", sa.Text(), nullable=True),
        sa.Column("is_active", sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_user_is_active"), "user", ["is_active"], unique=False)
    op.create_index(op.f("ix_user_username"), "user", ["username"], unique=True)
    op.create_table(
        "contact",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=True),
        sa.Column("contact_id", sa.Integer(), nullable=True),
        sa.Column("is_active", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(["contact_id"], ["user.id"], onupdate="CASCADE", ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["user_id"], ["user.id"], onupdate="CASCADE", ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("user_id", "contact_id"),
    )
    op.create_index(op.f("ix_contact_is_active"), "contact", ["is_active"], unique=False)
    op.create_table(
        "message",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("contact_id", sa.Integer(), nullable=False),
        sa.Column("text", sa.Text(), nullable=True),
        sa.Column("timestamp", sa.DateTime(), nullable=True),
        sa.Column("is_delivered", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(["contact_id"], ["contact.id"], onupdate="CASCADE", ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_message_is_delivered"), "message", ["is_delivered"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_message_is_delivered"), table_name="message")
    op.drop_table("message")
    op.drop_index(op.f("ix_contact_is_active"), table_name="contact")
    op.drop_table("contact")
    op.drop_index(op.f("ix_user_username"), table_name="user")
    op.drop_index(op.f("ix_user_is_active"), table_name="user")
    op.drop_table("user")
    op.drop_table("history")
    # ### end Alembic commands ###



# Form implementation generated from reading ui file './server/gui/server_gui_clients.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtWidgets


class Ui_ClientsWindow(object):
    def setupUi(self, ClientsWindow):
        ClientsWindow.setObjectName("ClientsWindow")
        ClientsWindow.resize(429, 466)
        self.verticalLayout = QtWidgets.QVBoxLayout(ClientsWindow)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tableWidget = QtWidgets.QTableWidget(ClientsWindow)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(0)
        self.tableWidget.setRowCount(0)
        self.verticalLayout.addWidget(self.tableWidget)

        self.retranslateUi(ClientsWindow)
        QtCore.QMetaObject.connectSlotsByName(ClientsWindow)

    def retranslateUi(self, ClientsWindow):
        _translate = QtCore.QCoreApplication.translate
        ClientsWindow.setWindowTitle(_translate("ClientsWindow", "Список клиентов"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    ClientsWindow = QtWidgets.QWidget()
    ui = Ui_ClientsWindow()
    ui.setupUi(ClientsWindow)
    ClientsWindow.show()
    sys.exit(app.exec())



# Form implementation generated from reading ui file './server/gui/server_gui_history.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtWidgets


class Ui_HistoryWindow(object):
    def setupUi(self, HistoryWindow):
        HistoryWindow.setObjectName("HistoryWindow")
        HistoryWindow.resize(618, 480)
        self.verticalLayout = QtWidgets.QVBoxLayout(HistoryWindow)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tableWidget = QtWidgets.QTableWidget(HistoryWindow)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(0)
        self.tableWidget.setRowCount(0)
        self.verticalLayout.addWidget(self.tableWidget)

        self.retranslateUi(HistoryWindow)
        QtCore.QMetaObject.connectSlotsByName(HistoryWindow)

    def retranslateUi(self, HistoryWindow):
        _translate = QtCore.QCoreApplication.translate
        HistoryWindow.setWindowTitle(_translate("HistoryWindow", "История клиентов"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    HistoryWindow = QtWidgets.QWidget()
    ui = Ui_HistoryWindow()
    ui.setupUi(HistoryWindow)
    HistoryWindow.show()
    sys.exit(app.exec())



import argparse
import sys
import warnings

from common.decorators import log
from server.config import ServerConf
from server.logger_conf import call_logger, main_logger
from server.transport import JIMServer


@log(call_logger)
def parse_args():
    parser = argparse.ArgumentParser(
        description="Launch JSON instant messaging (JIM) server.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-a",
        "--address",
        help="IP address for server listener",
        type=type(ServerConf.DEFAULT_LISTENER_ADDRESS),
        default=ServerConf.DEFAULT_LISTENER_ADDRESS,
    )
    parser.add_argument(
        "-p",
        "--port",
        help="Port for server listener",
        type=type(ServerConf.DEFAULT_PORT),
        default=ServerConf.DEFAULT_PORT,
    )
    args = parser.parse_args()
    if not 1023 < args.port < 65536:
        main_logger.critical(
            f"Попытка запуска сервера с неподходящим номером порта: {args.port}. Допустимы адреса с 1024 до 65535."
        )
        sys.exit(1)
    return args.address, args.port


def run():
    main_logger.info("Приложение запущено.")
    try:
        ip, port = parse_args()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            with JIMServer(ip, port) as jim_server:
                jim_server.start_server()
    except KeyboardInterrupt:
        main_logger.info("Работа сервера была принудительно завершена.")
        sys.exit(0)
    except Exception as ex:
        main_logger.critical(str(ex))
        sys.exit(1)


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['server', 'server.gui', 'server.migrations', 'server.migrations.versions']

package_data = \
{'': ['*']}

install_requires = \
['PyQt6>=6.4.0,<7.0.0',
 'SQLAlchemy>=1.4.44,<2.0.0',
 'alembic>=1.8.1,<2.0.0',
 'gb-pychat-common>=0.1.0,<0.2.0']

entry_points = \
{'console_scripts': ['server-cli = server.run_server_cli:run',
                     'server-gui = server.run_server_gui:run']}

setup_kwargs = {
    'name': 'gb-pychat-server',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'DashViolin',
    'author_email': 'mymail@srv.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import logging
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler

from server.config import ServerConf

main_logger = logging.getLogger("server.main")
main_formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(module)s - %(message)s")
main_logger.setLevel(logging.DEBUG)

file_handler = TimedRotatingFileHandler(
    filename=ServerConf.MAIN_LOG_FILE_PATH,
    encoding=ServerConf.ENCODING,
    when="D",
    interval=1,
    backupCount=7,
)
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(main_formatter)
main_logger.addHandler(file_handler)

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)
console_handler.setFormatter(main_formatter)
main_logger.addHandler(console_handler)


call_logger = logging.getLogger("server.calls")
call_logger.setLevel(logging.DEBUG)
call_handler = RotatingFileHandler(
    filename=ServerConf.CALL_LOG_FILE_PATH,
    maxBytes=1024 * 100000,  # 100000 KiB
    backupCount=7,
    encoding=ServerConf.ENCODING,
)
call_formatter = logging.Formatter("%(asctime)s - %(message)s")
call_handler.setFormatter(call_formatter)
call_handler.setLevel(logging.DEBUG)
call_logger.addHandler(call_handler)



from contextlib import ContextDecorator
from http import HTTPStatus
from ipaddress import ip_address
from select import select
from socket import AF_INET, SOCK_STREAM, socket
from time import sleep

from common.base import JIMBase
from common.decorators import login_required
from common.descriptors import PortDescriptor
from common.errors import IncorrectDataRecivedError, NonDictInputError, ReqiuredFieldMissingError
from common.meta import JIMMeta
from common.schema import Actions, Keys
from server.config import ServerConf
from server.logger_conf import main_logger
from server.storage import ServerStorage


class JIMServer(JIMBase, ContextDecorator, metaclass=JIMMeta):
    """
    Класс ядра сервера, отвечает за роутинг сообщений и взаимодействие с клиентами и базой данных
    """

    port = PortDescriptor()

    def __init__(self, ip: str, port: int | str) -> None:
        super().__init__()
        self.ip = ip_address(ip)
        self.port = port
        self.sock = socket(AF_INET, SOCK_STREAM)
        self.connections = []
        self.active_clients = dict()
        self.storage = ServerStorage()
        self.is_running = False

    def __str__(self):
        return "JIM_server_object"

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        main_logger.info("Закрываю соединение...")
        self.close()

    def _listen(self):
        self.sock.bind((str(self.ip), self.port))
        self.sock.setblocking(False)
        self.sock.settimeout(0.2)
        self.sock.listen(ServerConf.MAX_CONNECTIONS)
        main_logger.info(f"Сервер запущен на {self.ip}:{self.port}.")
        self.is_running = True

    def start_server(self):
        """Запускает сервер на прослушивание порта и обработку сообщений"""
        while True:
            try:
                self._listen()
                break
            except OSError:
                main_logger.info(f"Ожидается освобождение сокета {self.ip}:{self.port}...")
                sleep(1)
        self._mainloop()

    def _mainloop(self):
        while True:
            if not self.is_running:
                break
            try:
                conn, addr = self.sock.accept()
            except OSError:
                pass
            else:
                main_logger.info(f"Подключился клиент {':'.join(map(str, addr))}")
                self.connections.append(conn)
            finally:
                r_clients = []
                try:
                    if self.connections:
                        r_clients, _, _ = select(self.connections, self.connections, self.connections)
                except OSError:
                    pass
                for client in r_clients:
                    self._accept_message(client)
                self._cleanup_disconnected_users()

    def _cleanup_disconnected_users(self):
        disconnected_users = []
        for user, conn in self.active_clients.items():
            try:
                conn.getpeername()
            except OSError:
                disconnected_users.append((conn, user))
        for conn, user in disconnected_users:
            self._disconnect_client(conn, user)

    def _disconnect_client(self, conn: socket, user: str | None = None):
        if user:
            self.storage.change_user_status(username=user, is_active=False)
            try:
                self.active_clients.pop(user)
            except KeyError:
                pass
        try:
            self.connections.remove(conn)
        except ValueError:
            pass
        conn.close()
        main_logger.info(f"Клиент отключился.")

    def close(self):
        self.sock.close()
        self.connections = []
        self.active_clients = dict()
        self.is_running = False

    def _accept_message(self, client_conn: socket):
        response_code = HTTPStatus.OK
        response_descr = ""
        disconnect_client = False
        msg = self._recv(client_conn)
        if msg:
            try:
                self._validate_msg(msg)
                main_logger.debug(f"Принято сообщение: {msg}")
                if msg.get(Keys.ACTION) == Actions.AUTH:
                    response_code, response_descr, disconnect_client = self._register_user(msg, client_conn)
                else:
                    response_code, response_descr, disconnect_client = self._route_msg(msg, client_conn)
            except (NonDictInputError, IncorrectDataRecivedError) as ex:
                response_code = HTTPStatus.INTERNAL_SERVER_ERROR
                response_descr = str(ex)
                main_logger.error(f"Принято некорректное сообщение: {response_descr}")
            except ReqiuredFieldMissingError as ex:
                response_code = HTTPStatus.INTERNAL_SERVER_ERROR
                response_descr = str(ex)
                main_logger.error(f"Ошибка валидации сообщения: {response_descr}")
            except Exception as ex:
                response_code = HTTPStatus.INTERNAL_SERVER_ERROR
                response_descr = str(ex)
                main_logger.error(f"Непредвиденная ошибка: {response_descr}")
            finally:
                response = self._make_response_msg(code=response_code, description=response_descr)
                self._send(msg=response, client=client_conn)
                main_logger.debug(f"Отправлен ответ: {response}")
                if disconnect_client:
                    self._disconnect_client(client_conn)

    def _register_user(self, msg: dict, client_conn: socket):
        username = msg[Keys.USER][Keys.ACCOUNT_NAME]
        if username in self.active_clients:
            response_code = HTTPStatus.FORBIDDEN
            response_descr = "Клиент с таким именем уже зарегистрирован на сервере"
            disconnect_client = True
        else:
            passwd = msg[Keys.USER].get(Keys.PASSWORD)
            ip = client_conn.getpeername()[0]
            if self.storage.check_user_auth(username=username, password=passwd):
                self.active_clients[username] = client_conn
                self.storage.register_user_login(username=username, ip_address=ip)
                response_code = HTTPStatus.OK
                response_descr = ""
                disconnect_client = False
            else:
                response_code = HTTPStatus.FORBIDDEN
                response_descr = "Неверное имя пользователя или пароль"
                disconnect_client = True
        return response_code, response_descr, disconnect_client

    @login_required
    def _route_msg(self, msg: dict, client_conn: socket):
        response_code = HTTPStatus.OK
        response_descr = ""
        disconnect_client = False
        match msg.get(Keys.ACTION):
            case Actions.PRESENCE:
                username = msg[Keys.USER][Keys.ACCOUNT_NAME]
                status = msg[Keys.USER].get(Keys.STATUS)
                self.storage.change_user_status(username=username, status=status, is_active=True)
            case Actions.MSG:
                username = msg[Keys.FROM]
                target_user = msg[Keys.TO]
                self.storage.add_contact(username=target_user, contact_name=username)
                if target_user in self.active_clients:
                    client_conn = self.active_clients[target_user]
                    self._send(msg=msg, client=client_conn)
                    main_logger.debug(f"Отправлено сообщение: {msg}")
                else:
                    response_code = HTTPStatus.NOT_FOUND
                    response_descr = "Пользователь не активен"
            case Actions.CONTACTS:
                username = msg[Keys.ACCOUNT_NAME]
                contacts = self.storage.get_user_contacts(username=username)
                response_code = HTTPStatus.ACCEPTED
                response_descr = contacts
            case Actions.ADD_CONTACT:
                username = msg[Keys.ACCOUNT_NAME]
                contact = msg[Keys.CONTACT]
                ok = self.storage.add_contact(username=username, contact_name=contact)
                if not ok:
                    response_code = HTTPStatus.NOT_FOUND
            case Actions.DEL_CONTACT:
                username = msg[Keys.ACCOUNT_NAME]
                contact = msg[Keys.CONTACT]
                ok = self.storage.delete_contact(username=username, contact_name=contact)
                if not ok:
                    response_code = HTTPStatus.NOT_FOUND
            case Actions.QUIT:
                disconnect_client = True
            case Actions.JOIN | Actions.LEAVE:
                response_code = HTTPStatus.BAD_REQUEST
            case _:
                response_code = HTTPStatus.BAD_REQUEST
        return response_code, response_descr, disconnect_client

    def _recv(self, client) -> dict | None:
        msg = None
        try:
            raw_data = client.recv(self.package_length)
            msg = self._load_msg(raw_data)
        except (IncorrectDataRecivedError, OSError, ConnectionRefusedError, ConnectionResetError, BrokenPipeError):
            self._disconnect_client(conn=client)
        finally:
            return msg

    def _send(self, msg, client):

        msg_raw_data = self._dump_msg(msg)
        client.send(msg_raw_data)

    def _make_probe_msg(self):
        msg = {Keys.ACTION: Actions.PROBE}
        self._update_timestamp(msg=msg)
        return msg

    def _make_response_msg(self, code: HTTPStatus, description: str | list = ""):
        if not isinstance(description, list):
            description = description or code.phrase
        msg = {
            Keys.RESPONSE: code.value,
            Keys.ERROR if 400 <= code.value < 600 else Keys.ALERT: description,
        }
        self._update_timestamp(msg=msg)
        return msg
